<?php

namespace Drupal\styled_image\Plugin\Field\FieldWidget;


use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Plugin\Field\FieldWidget\ImageWidget;

/**
 * Plugin implementation of the 'styled_image_widget' widget.
 *
 * @FieldWidget(
 *   id = "styled_image_widget",
 *   label = @Translation("Image with style"),
 *   field_types = {
 *     "styled_image"
 *   }
 * )
 */
class StyledImageWidget extends ImageWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'image_style_type' => 'image_style',
      'image_styles' => [],
      'responsive_image_styles' => [],
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state); // TODO: Change the autogenerated stub

    $elements['image_style_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Image style type'),
      '#options' => self::getImageStyleTypes(),
      '#default_value' => $this->getSetting('image_style_type'),
    ];

    $elements['image_styles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Image styles'),
      '#options' => self::getImageStyleOptions('image_style'),
      '#default_value' => $this->getSetting('image_styles'),
      '#states' => [
        'disabled' => [
          ':input[name$="[image_style_type]"]' => [
            'value' => 'responsive_image_style',
          ],
        ],
        'invisible' => [
          ':input[name$="[image_style_type]"]' => [
            'value' => 'responsive_image_style',
          ],
        ],
      ],
    ];

    $elements['responsive_image_styles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Image styles'),
      '#options' => self::getImageStyleOptions('responsive_image_style'),
      '#default_value' => $this->getSetting('responsive_image_styles'),
      '#states' => [
        'disabled' => [
          ':input[name$="[image_style_type]"]' => [
            'value' => 'image_style',
          ],
        ],
        'invisible' => [
          ':input[name$="[image_style_type]"]' => [
            'value' => 'image_style',
          ],
        ],
      ],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary(); // TODO: Change the autogenerated stub

    switch ($this->getSetting('image_style_type')) {
      case 'image_style':
        $image_style_type = $this->t('Image styles');
        $image_styles = self::getImageStyleOptions(
          $this->getSetting('image_style_type'),
          $this->getSetting('image_styles')
        );
        break;
      case 'responsive_image_style':
        $image_style_type = $this->t('Responsive image styles');
        $image_styles = self::getImageStyleOptions(
          $this->getSetting('image_style_type'),
          $this->getSetting('responsive_image_styles')
        );
        break;
    }

    $summary[] = $this->t('Available @image_style_type: @image_styles', [
      '@image_style_type' => $image_style_type,
      '@image_styles' => implode(', ', $image_styles),
    ]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $elements = parent::formElement($items, $delta, $element, $form, $form_state);

    $elements['image_style_type'] = [
      '#type' => 'value',
      '#value' => $this->getSetting('image_style_type'),
    ];

    $options = [];

    switch ($this->getSetting('image_style_type')) {
      case 'image_style':
        $options = self::getImageStyleOptions(
          $this->getSetting('image_style_type'),
          $this->getSetting('image_styles')
        );
        break;
      case 'responsive_image_style':
        $options = self::getImageStyleOptions(
          $this->getSetting('image_style_type'),
          $this->getSetting('responsive_image_styles')
        );
        break;
    }

    if ($options) {
      $elements['image_style'] = [
        '#type' => 'select',
        '#title' => $this->t('Image style'),
        '#access' => (bool) $elements['#default_value']['fids'],
        '#options' => $options,
        '#require' => TRUE,
	      '#default_value' => $elements['#default_value']['image_style'],
      ];
    }

    return $elements;
  }

  /**
   * Gets the (responsive) image style options.
   *
   * @param $image_stype_type
   *   Either 'image_style' or 'responsive_image_style'
   * @param array $filter
   *   An array of image style ids to filter the output array.
   *
   * @return array
   *    Returns an array of image style names keyed by the id.
   */
  private static function getImageStyleOptions($image_stype_type, array $filter = []) {
    $options = [];

    $styles = \Drupal::entityTypeManager()
      ->getStorage($image_stype_type)
      ->loadMultiple();

    foreach ($styles as $style) {
      $options[$style->id()] = $style->label();
    }

    if ($filter) {
      $options = array_intersect_key($options, array_filter($filter));
    }

    return $options;
  }

  /**
   * Gets the available image style type options.
   *
   * @return array
   *   The available image style type names keyed by the entity tpye id
   */
  private static function getImageStyleTypes() {
    $options = [];

    $options['image_style'] = t('Image style');

    if (\Drupal::moduleHandler()->moduleExists('responsive_image')) {
      $options['responsive_image_style'] = t('Responsive image style');
    }

    return $options;
  }
}
